*  *
* FUNCTION K_SCROLL
* Parameters:
*     parent_dbf   alias of the parent file
*     relate_key   key into the child file
*     child_dbf    alias of child file
*     fall_out     if passed and true, return after display
*     fields       if passed, array of names of fields to
*                  display, otherwise all fields up to 64
*     pictures     if passed, array of PICTURE strings
*     heads        if passed, array of DBEDIT() headers
*     ed_permits   if passed, array of edit permissions
*     headliner    if passed, box header, else ALIAS()
*     tr           if passed, top row of window,
*                  otherwise MIN( cur_row + 1, 15 )
*     tc           if passed, left column, else 2
*     br           if passed and numeric, bottom row,
*                  otherwise K_SCROLL() calculates
*     bc           if passed, right column for window, else 77
*     udf_kbrow    if passed, inner udf, else "KSCR_UDF"
*     udf_except   if passed, exception handler for KSCR_UDF,
*                  otherwise KEXCEPT()
*     udf_xedit    if passed, field edit func, else K_XEDIT().
*     color        if passed, window color, else current color
*  *


*  *
FUNCTION K_SCROLL
PARAMETERS parent_dbf, relate_key, child_dbf, fall_out, fields, ;
           pictures, heads, ed_permits, headliner, tr, tc, br, ;
           bc, udf_kbrow, udf_except, udf_xedit, color
*  *
PRIVATE cur_row, result, cur_order, new_order, _kcount, _key, ;
        _lastkey, key_count, ra_size, _curclr, _firstrec, ;
        _lastrec, _toprec, _bottrec, _toprow, _botrow, ;
        _winsize, _thisrow, _thiscol, k_screen, is_append

cur_row = ROW()
result  = 0
_curclr = SETCOLOR()

SELECT ( SELECT( child_dbf ))        && DBEDIT() can scroll
ra_size   = MIN( FCOUNT(), 64 )      && only 64 fields.
cur_order = INDEXORD()               && Save and set index order.
SET ORDER TO GET_ORDER( relate_key )

IF TYPE( "fields" ) != "A"           && If fields[] not passed,
    PRIVATE fields[ ra_size ]        && declare and initialize.
    AFIELDS( fields )
ENDIF

IF TYPE( "pictures" ) != "A"         && Same for pictures[].
    PRIVATE pictures[ ra_size ]
    APICS( pictures, fields )        && Support routine.
ENDIF

IF TYPE( "heads" ) != "A"            && Same for headers[].
    PRIVATE heads[ ra_size ]         
    AHEADS( fields, heads )          && Support routine.
ENDIF

IF TYPE( "ed_permits" ) != "A"       && Same for ed_permits[].
   PRIVATE ed_permits[ ra_size ]
   AFILL( ed_permits, .T. )
ENDIF

fall_out   = IF( TYPE( "fall_out" ) != "L", .F., fall_out )
headliner  = IF( TYPE( "headliner" ) != "C", ALIAS(), headliner )
udf_kbrow  = IF( TYPE( "udf_kbrow" ) != "C", ;
                      "KSCR_UDF", udf_kbrow )
udf_except = IF( TYPE( "udf_except" ) != "C", ;
                      "K_EXCEPT(_lastkey)", udf_except )
udf_xedit  = IF( TYPE( "udf_xedit" ) != "C", ;
                      "K_XEDIT()", udf_xedit )
color      = IF( TYPE( "color" ) != "C", SETCOLOR(), color )

cur_parent = &parent_dbf->( &relate_key )
_kcount    = 0
SEEK cur_parent

IF FOUND()

    _firstrec = RECNO()
    DO WHILE &relate_key = cur_parent .AND. !EOF()
       _lastrec = RECNO()
       _kcount  = _kcount + 1
       SKIP
    ENDDO
    GO _firstrec

    * If no location passed, try just below the cursor
    tr = IF( TYPE( "tr" ) != "N", MIN( cur_row + 1, 15 ), tr )
    br = IF( TYPE( "br" ) != "N", MIN( 23, tr + _kcount + 2 ), br )
    tc = IF( TYPE( "tc" ) != "N", 2, tc )
    bc = IF( TYPE( "bc" ) != "N", 77, bc )

    IF fall_out
        KEYBOARD CHR( 27 )
    ELSE
        k_screen = SAVESCREEN( tr-1, tc-1, br+1, bc+1 )
    ENDIF

    _lastkey = 0                    && Last key pressed.
    _key     = 0                    && Inner key signal for us.

    _toprow  = tr + 2               && Top data row of window.
    _botrow  = br - 1               && Bottom data row of window.
    _winsize = br - tr - 2          && No. of window data rows.
    _thisrow = _toprow              && Current window row.

    _toprec  = 1                    && First record in window.
    _bottrec = _winsize             && Last record in window.

    SET KEY  5 TO K_NTRCPT          && Up
    SET KEY 24 TO K_NTRCPT          && Dn
    SET KEY 18 TO K_NTRCPT          && PgUp
    SET KEY  3 TO K_NTRCPT          && PgDn
    SET KEY 31 TO K_NTRCPT          && Ctrl-PgUp
    SET KEY 30 TO K_NTRCPT          && Ctrl-PgDn

    SETCOLOR( color )
    @ tr - 1, tc, br, bc BOX "͸Գ "
    headliner = "  " + headliner + "  "
    @ tr, CENTRE( headliner, tc, bc - tc ) SAY headliner
    DBEDIT( tr, tc+1, br, bc-1, fields, udf_kbrow, ;
            pictures, heads, .F., .F., "", .F. )
    IF LASTKEY() = 27 .AND. !fall_out
        RESTSCREEN( tr-1, tc-1, br+1, bc+1, k_screen )
    ELSE
        result = 1
    ENDIF

ENDIF

SET KEY 24 TO                       && Restore initial state.
SET KEY  5 TO
SET KEY 18 TO
SET KEY  3 TO
SET KEY 30 TO
SET KEY 31 TO
SETCOLOR( _curclr )
SET ORDER TO cur_order              
SELECT ( SELECT( parent_dbf ))

RETURN result



*  *
FUNCTION K_NTRCPT
*  *
_lastkey = LASTKEY()                && Declared by K_SCROLL().
_key     = 0                        && Ditto.

DO CASE

    CASE _lastkey = 5               && Up
        IF _thisrow > _toprow .OR. _toprec > 1
            _key     = 5
            IF _thisrow == _toprow
                _toprec  = _toprec - 1
                _bottrec = _bottrec - 1
            ELSE
                _thisrow = _thisrow - 1
            ENDIF
        ENDIF

    CASE _lastkey = 24              && Dn
        IF _thisrow < _botrow .OR. _bottrec < _kcount
            _key = 24
            IF _thisrow == _botrow
                _bottrec = _bottrec + 1
                _toprec  = _toprec + 1
            ELSE
                _thisrow = _thisrow + 1
            ENDIF
        ENDIF

    CASE _lastkey = 18              && PgUp
        IF _toprec > _winsize
            _key     = 18
            _toprec  = _toprec - _winsize
            _bottrec = _bottrec - _winsize
        ELSE
            KEYBOARD CHR( 1 )       && Message to KSCR_UDF()
            _key = -996
        ENDIF

    CASE _lastkey = 3               && PgDn
        IF ( _bottrec + _winsize ) <= _kcount
            _key     = 3
            _toprec  = _toprec + _winsize
            _bottrec = _bottrec + _winsize
        ELSE
            KEYBOARD CHR( 1 )       && Message to KSCR_UDF()
            _key = -997
        ENDIF

    CASE _lastkey = 31              && Ctrl-PgUp
        KEYBOARD CHR( 1 )           && Message to KSCR_UDF()
        _key = -998                 

    CASE _lastkey = 30              && Ctrl-PgDn
        KEYBOARD CHR( 1 )           && Message to KSCR_UDF()
        _key = -999

ENDCASE

IF _key > 0                         && Suspend key interception
    SET KEY 24 TO                   && so DBEDIT() can process
    SET KEY  5 TO                   && the movement key.
    SET KEY 18 TO
    SET KEY  3 TO
    SET KEY 30 TO
    SET KEY 31 TO
    KEYBOARD CHR( _key ) + ;        && Generate an exception to
             CHR( 254 )             && force a call to KSCR_UDF().
ENDIF

RETURN _key



*  *
FUNCTION KSCR_UDF
PARAMETERS _mode, fld_ptr
*  *
PRIVATE _result, cur_fld

_result    = 1                      && No-repaint.
cur_fld    = fields[ fld_ptr ]      && Inherited from K_SCROLL()
is_append  = .F.
_thisrow   = ROW()
_thiscol   = COL()
_lastkey  = LASTKEY()

IF _mode = 0 .AND. _lastkey = 1     && Signal for go top/bott.

    KEYBOARD CHR( 0 )

    IF _key = -996                  && Too high for Ctrl_PgUp,
                                    && so move to top row.
        SKIP _toprow - _thisrow

    ELSEIF _key = -997              && Too low for Ctrl-PgDn,
                                    && so move to bottom row.
        SKIP _botrow - _thisrow

    ELSEIF _key == -998             && Ctrl-PgUp

        GO _firstrec                && Re-position and repaint.
        SKIP _thisrow - _toprow
        _toprec  = 1
        _bottrec = _winsize
        _result  = 2                && Do-repaint.

    ELSEIF _key == -999             && Ctrl-PgDn.

        GO _lastrec                 && Re-position and repaint.
        SKIP _thisrow - _botrow
        _toprec  = _kcount - _winsize + 1
        _bottrec = _kcount
        _result  = 2                && Do-repaint.

    ENDIF

ELSEIF _mode = 4

    _result = &udf_except           && Default handler K_EXCEPT()

ENDIF

SET KEY 24 TO K_NTRCPT              && Turn the keyboard filter
SET KEY  5 TO K_NTRCPT              && back on.
SET KEY 18 TO K_NTRCPT
SET KEY  3 TO K_NTRCPT
SET KEY 30 TO K_NTRCPT
SET KEY 31 TO K_NTRCPT
_key = 0

RETURN _result



*  *
FUNCTION K_EXCEPT                      && Exception handler
PARAMETER _lastkey                     && for K_SCROLL().
*  *
PRIVATE _result, no_repaint, do_repaint, ins_blank, out, ;
        row, col

out        =  0
no_repaint =  1
do_repaint =  2
ins_blank  =  3
_result    = no_repaint
row        = ROW()                   && udf_xedit inherits
col        = COL()                   && ditto

DO CASE

    CASE _lastkey = 27               && ESCAPE: Exit.
        _result = out

    CASE _lastkey = 13               && ENTER: Edit.
        is_append = .F.              
        _result = &udf_xedit

ENDCASE

RETURN _result



*  *
FUNCTION K_XEDIT                     && Editor for K_SCROLL().
*  *
PRIVATE memvar, _result, _key, _scr

_key    = &relate_key                && Pre-edit key value
_result = no_repaint
IF TYPE( cur_fld ) != "M" .AND. ed_permits[ fld_ptr ]
   _scr = SAVESCREEN( 24, 0, 24, 79 )
   @ 24, 0 SAY "Editing" + SPACE( 73 )
   memvar = &cur_fld
   @ row, col GET memvar PICTURE pictures[ fld_ptr ]
   READ
   RESTSCREEN( 24, 0, 24, 79, _scr )
   IF LASTKEY() != 27
      REPLACE &cur_fld WITH memvar
      IF _key != &relate_key         && If the key is changed,
          _result = out              && force a new call of
          KEYBOARD CHR( 13 )         && K_SCROLL().
      ELSE
          _result = do_repaint
      ENDIF
   ENDIF
ELSE
   KEYBOARD CHR( 4 )                 && Arrow past no-edit field
   _result = do_repaint
ENDIF

RETURN _result



*  *
* Support routines for K_SCROLL()
*  *

*  *
FUNCTION APICS                         && Make picture strings
PARAMETERS p_ra, f_ra                  && for a field array.
*  *
PRIVATE i, max_len

max_len = MIN( LEN( p_ra ), LEN( f_ra ) )
FOR i = 1 TO max_len
    p_ra[ i ] = GEN_PIC( f_ra[ i ] )
NEXT

RETURN .T.



*  *
FUNCTION GEN_PIC                       && General purpose
PARAMETER _field                       && picture maker
*  *
PRIVATE pict_str, phone, caps, dec_pos, str_num, _fieldtype

_field     = UPPER( _field )
_fieldtype = TYPE( _field )
phone      = "@R (###) 999-9999"
caps       = "@!"

DO CASE
    CASE _fieldtype = "C"
        DO CASE
            CASE "PHONE" $ _field
                pict_str = phone
            CASE "CITY" $ _field
                pict_str = "!" + ;
                           REPLICATE( "X", LEN( &_field ) - 1 )
            CASE "_NO" $ _field
                pict_str = "@9"
            CASE LEN( &_field ) < 10
                pict_str = caps
            OTHERWISE
                pict_str = "@KS" + ;
                           LTRIM( STR( MIN( LEN( &_field), 60 )))
        ENDCASE
    CASE _fieldtype = "L"
        pict_str = "Y"
    CASE _fieldtype = "N"
        str_num = STR( &_field )
        dec_pos = AT( ".", str_num )
        IF dec_pos > 0
            pict_str = REPLICATE( "9", dec_pos - 1 ) + "." + ;
                       REPLICATE( "9", LEN( str_num ) - dec_pos )
        ELSE
            pict_str = REPLICATE( "9", LEN( str_num ))
        ENDIF
    OTHERWISE
        pict_str = ""
ENDCASE

RETURN pict_str




*  *
FUNCTION AHEADS                        && Make proper-case
PARAMETERS fld_array, hdr_array        && headers.
*  *
PRIVATE i, j, s

j = LEN( fld_array )

FOR i  = 1 TO j
    s = fld_array[ i ]
    hdr_array[ i ] = PROPER( STRTRAN( s, "_", " " ))
NEXT

RETURN .T.



*  *
FUNCTION GET_ORDER                     && Return the order of
PARAMETER _key, _alias                 && an index key.
*  *
PRIVATE args, cur_alias, i, s, _result

args      = PCOUNT()
cur_alias = ALIAS()
_key      = UPPER( TRIM( _key ))
_result   = 0
IF args > 1
    SELECT ( SELECT( _alias ))
ENDIF

i = 1
DO WHILE .T.
    s = INDEXKEY( i )
    IF EMPTY( s )
        EXIT
    ENDIF
    IF s = _key
        _result = i
        EXIT
    ENDIF
    i = i + 1
ENDDO

SELECT ( SELECT( cur_alias ))

RETURN _result



*  *
FUNCTION CENTRE                        && Find the column where
PARAMETERS _string, offset, _width     && a string is centred.
*  *
_width  = IF( TYPE( "_width" ) != "N", 80, _width )
_offset = IF( TYPE( "_offset" ) != "N", 0, _offset )

RETURN MAX( ( _width - LEN( _string ) ) / 2 + _offset, 0 )



*  *
FUNCTION PROPER                        && Proper-case a string.
PARAMETER _string
*  *
PRIVATE i, c, _length, _bool

_string = LOWER( _string )
_length = LEN( _string )
_bool   = .T.

FOR i = 1 TO _length
    c = SUBSTR( _string, i, 1 )
    IF _bool
        IF ISALPHA( c )
            _string = LEFT( _string, i - 1 ) + ;
                     UPPER( c ) + ;
                     RIGHT( _string, _length - i )
            _bool = .F.
        ENDIF
    ELSEIF c $  " ,.(){}[]-_!@;:?/"
        _bool = .T.
    ENDIF
NEXT

RETURN _string


*  *
* EOF KSCRL.PRG                       Sun  06-03-1990  12:39:21
*  *

