$COMMAND
srchname
*(SRCHNAME.APP                      by Paul Litwin         07/29/90  )
*(For R:BASE versions 2.11 and 3.0.                                  )
*(This application requests a last name, calls the SOUNDEX.CMD       )
*(routine to convert the name to its Soundex code, and presents the  )
*(user with a scrollable list of all similarly sounding names in the )
*(database.  It is set up to work with the PATIENT table of the      )
*(CLINIC database.  It could easily be adapted to work with other    )
*(databases where records are primarily located by last name and     )
*(misspellings may occur.                                            )
 
*(The PATIENT table of the CLINIC database could be created with the )
*(following command:                                                 )
*(CREATE TABLE PATIENT idnum INTEGER, firname TEXT 20, +             )
*(  lasname TEXT 20, address TEXT 40, city TEXT 20, state TEXT 2, +  )
*(  zipcode INTEGER, areacode TEXT 3, phonenum TEXT 8, age INTEGER, +)
*(  sex TEXT 1, soundex TEXT 4                                       )
 
*(Local variables:                                                   )
*(    vblank   = blank text field used to position SELECT command.   )
*(    vlname   = surname to be searched for.                         )
*(    vlen     = length of vlname.                                   )
*(    vmatch   = number of records found with same Soundex code.     )
*(    vcnt1    = first record# of matched records to list on screen. )
*(    vcnt2    = last record# of matched records to list on screen.  )
*(    vtscrn   = total # of screens needed for matched records with  )
*(               18 records per screen.                              )
*(    vscrn    = currently displayed screen number.                  )
*(    vaction  = response of user to FILLIN command.                 )
*(    vid      = ID# selected by user.                               )
 
*(Routines called:                                                   )
*(    soundex.cmd  : Routine to compute soundex code.  See Sept '90  )
*(                   issue of Data Based Advisor.                    )
*(                   Variables passed: vlname, vlen.                 )
 
 
SET QUOTE ="
LABEL startcmd
 
CLS
SET V vblank TEXT +
      vlname TEXT +
      vlen INTEGER +
      vmatch INTEGER +
      vcnt1 INTEGER +
      vcnt2 INTEGER +
      vtscrn INTEGER +
      vscrn INTEGER +
      vaction TEXT +
      vid INTEGER
 
SET V vblank = " "
SET HEADINGS OFF
SET WIDTH 85
SET MESSAGES OFF
SET ERROR MESSAGES OFF
SET ESC OFF
SET BELL OFF
 
*(Request last name and compute length.                              )
DISPLAY getname IN srchname.apx AT 16
FILLIN vlname USING "[" AT 18,50
SET V vlen TO (SLEN(.vlname))
 
*(If length = 0, then null repsonse, so quit routine.                )
IF vlen = 0 THEN
  GOTO abortcmd
ENDIF
 
*(Compute soundex code for vlname. Use SOUNDEX3.CMD for R:BASE 3.0.  )
RUN soundex.cmd
 
*(Compute the number of records with the same soundex value.         )
CLS
WRITE "Standby while searching..." AT 1,40
COMPUTE vmatch AS COUNT idnum FROM patient WHERE soundex = .vsdx
 
*(If no matches found, then enter a new patient record.              )
IF vmatch = 0 THEN
  DISPLAY nomatch IN srchname.apx AT 13
  PAUSE
  ENTER ptvisit FOR 1 ROW
  GOTO startcmd
ENDIF
 
*(Since we found at least one match, we must now display the matches )
*(so the user can determine if any of the potential matches are      )
*(the patient we are looking for.                                    )
CLS
WRITE "Standby while loading matches..." AT 1,40
 
*(The temporary table CTEMP will hold the potential matches.  A      )
*(temporary table is used instead of a View for fast scrolling.      )
*(Prior to the Project, remove any old copy of this table.  The      )
*(columns used in the Project command are selected for their value   )
*(in helping the user to discriminate between potential matches.     )
*(The Project is sorted by first name, since the last name is        )
*(assumed to be equivalent -- remember that all matches share the    )
*(same Soundex code.                                                 )
REMOVE ctemp
PROJECT ctemp FROM patient USING idnum firname lasname address +
  city sex age +
  SORTED BY firname WHERE soundex = .vsdx
 
*(Initialize the vcnt# variables which will keep track of the 1st and)
*(last records to display on the screen.                             )
SET V vcnt1 TO 1 +
      vcnt2 TO 18
 
*(Vtscrn is computed to determine the number of 18 record screens    )
*(needed to display all the matches.                                 )
SET V vtscrn = (INT(.vmatch/18 + 0.999))
 
 
*(The While loop is used to display the records and scroll through   )
*(the matches as necessary.                                          )
WHILE #pi EXISTS THEN
  CLS
*(Vscrn is the current screen number.  It is determined by dividing  )
*(vcnt2 [the last record of the current screen] by 18 & rounding up. )
  SET V vscrn = (INT(.vcnt2/18 + 0.999))
 
*(Display the headings and some descriptive data of the matches.     )
  DISPLAY srchlst IN srchname.apx AT 1
  SHOW VAR vmatch=4 AT 1,3
  SHOW VAR vsdx=4 AT 1,28
  SHOW VAR vscrn=2 AT 1,70
  SHOW VAR vtscrn=2 AT 1,76
 
*(Vblank is used to moves the screen cursor to line 3, so that the   )
*(Select command will display on line 4.  The Select command displays)
*(up to 18 records using vcnt1 and vcnt2 to limit the records.       )
  SHOW VAR vblank=1 AT 3,79
  SELECT idnum=5 (SGET(firname,10,1))=10 (SGET(lasname,20,1))=20 +
    (SGET(address,22,1))=22 (SGET(city,10,1))=10 (LUC(sex))=1 age=3 +
     FROM ctemp WHERE COUNT GE .vcnt1 AND COUNT LE .vcnt2
 
*(Now request the next action from the user.  The user may scroll up )
*(or down, or enter Q to quit and continue.                          )
  DISPLAY srchact1 IN srchname.apx AT 22
  FILLIN vaction=0 USING "[" AT 23,76
 
*(The scroll-down routine.  Vcnt1 and vcnt2 are incremented by 18.   )
  IF vaction = "[PgDn]" OR vaction = "[Down]" THEN
 
    IF vscrn < .vtscrn THEN
      SET V vcnt1 TO (.vcnt1 + 18)
 
      IF vcnt1 > .vmatch THEN
        SET V vcnt1  TO (.vmatch - 17)
      ENDIF
 
      IF vcnt1 < 1 THEN
        SET V vcnt1 TO 1
      ENDIF
 
      SET V vcnt2 TO (.vcnt1 + 17)
 
    ENDIF
    GOTO endloop
  ENDIF
 
 
*(The scroll-up routine.  Vcnt1 and vcnt2 are decremented by 18.     )
  IF vaction = "[PgUp]" OR vaction = "[Up]" THEN
 
    IF vscrn > 1 THEN
      SET V vcnt1 TO (.vcnt1 - 18)
 
      IF vcnt1 < 1 THEN
       SET V vcnt1 TO 1
      ENDIF
 
      SET V vcnt2 TO (.vcnt1 + 17)
 
    ENDIF
    GOTO endloop
  ENDIF
 
 
*(If Q or enter are pressed, then break out of the while loop.       )
  IF LUC(vaction) = "Q" OR vaction = "[Enter]" THEN
    BREAK
  ENDIF
 
  LABEL endloop
ENDWHILE
 
 
*(Ask user if a match was found.  If so, have user enter ID#.        )
*(If not, the user enters 0 or a null response.                      )
CLS FROM 22 TO 24
DISPLAY matchid IN srchname.apx AT 22
FILLIN vid USING "[" AT 23,71
 
IF vid FAILS THEN
  SET V vid = 0
ENDIF
 
*(If no match, then enter a new PATIENT record.                      )
*(Otherwise, edit old record.                                        )
IF vid = 0 THEN
  ENTER ptvisit FOR 1 ROW
ELSE
  EDIT USING ptvisit WHERE idnum = .vid
ENDIF
 
*(Go back to beginning to request another patient name.              )
GOTO startcmd
 
 
LABEL abortcmd
 
*(Reset headings and width.                                          )
SET HEADINGS ON
SET WIDTH 79
CLEAR VAR vid vlname vmatch vlen vaction vcnt1 vcnt2 vscrn vtscrn vblank
RETURN
$SCREEN
getname
Ŀ
        Enter Last Name of patient                                      
        or leave blank to quit.                 [                    ]  

$SCREEN
nomatch
          ͻ
                                           
                  No matches found.        
                                           
          Hit any key to continueͼ
           
$SCREEN
srchlst
     soundex matches for     Screen    of   Ŀ
 ID#   FirstName  LastName             Address                City     Sex Age
       
$SCREEN
srchact1
Ŀ
 [PgDn] to scroll down     [PgUp] to scroll up     [Q]uit & continue      [Q]

$SCREEN
matchid
Ŀ
 Enter ID# of matched case,    or (0) if no match found              [0    ] 

